@@ -2,7 +2,7 @@
 #
 # The fisx library for X-Ray Fluorescence
 #
-# Copyright (c) 2014-2020 European Synchrotron Radiation Facility
+# Copyright (c) 2014-2023 European Synchrotron Radiation Facility
 #
 # This file is part of the fisx X-ray developed by V.A. Sole
 #
@@ -142,6 +142,54 @@ public:
     const std::map<std::string, double> & getRadiativeTransitions(const std::string & elementName, \
                                                                   const std::string & subshell) const;
 
+    /*!
+    Convenience method to simplify access to element calculations from binding (ex. python)
+
+    Given a set of energies, give the initial distribution of vacancies (before cascade) due to
+    photoelectric effect.
+    The output map keys correspond to the different partial photoelectric shells and the values
+    are just vectors of mu_photoelectric(shell, E)/mu_photoelectric(total, E)
+    */
+    std::map<std::string, std::vector <double> >getInitialPhotoelectricVacancyDistribution(\
+                                                const std::string & elementName,
+                                                const std::vector<double> & energies) const;
+
+    /*!
+    Convenience method to simplify access to element calculations from binding (ex. python)
+
+    Given one energy, give the initial distribution of vacancies (before cascade) due to
+    photoelectric effect.
+    The output map keys correspond to the different subshells and the values are just
+    mu_photoelectric(shell, E)/mu_photoelectric(total, E).
+    */
+    std::map<std::string, double> getInitialPhotoelectricVacancyDistribution(\
+                                                                             const std::string & elementName,
+                                                                             const double & energy) const;
+
+    std::map<std::string, double> getCascadeModifiedVacancyDistribution(const std::string & elementName,
+                                                                        const std::map<std::string, \
+                                                                        double> & distribution) const;
+
+    /*!
+    Convenience method to simplify access to element calculations from binding (ex. python)
+
+    Given an initial vacancy distribution, returns the emitted X-rays.
+
+    Input:
+    distribution - Map[key, double] of the form [(sub)shell][amount of vacancies]
+    cascade - Consider de-excitation cascade (default is 1 = true)
+    useFluorescenceYield - Correct by fluorescence yield (default is 1 = true)
+
+    Output:
+    map[key]["rate"] - emission rate where key is the transition line (ex. KL3)
+    map[key]["energy"] - emission energy where key is the transition line (ex. KL3)
+    */
+    std::map<std::string, std::map<std::string, double> >\
+        getXRayLinesFromVacancyDistribution(const std::string & elementName,
+                                            const std::map<std::string, double> & distribution, \
+                                            const int & cascade = 1,
+                                            const int & useFluorescenceYield = 1) const;
+
     // shell related functions
     /*!
     Convenience method to simplify access to element properties from binding (ex. python)
@@ -227,8 +275,9 @@ public:
     values are the vectors of the calculated values via log-log interpolation in the internal table.
     */
     std::map<std::string, std::vector<double> > getMassAttenuationCoefficients(\
-                                                std::map<std::string, double> elementMassFractions,\
-                                                std::vector<double> energies) const;
+                                                const std::map<std::string, double> & elementMassFractions,\
+                                                const std::vector<double> & energies,
+                                                const int & isComposition = 0) const;
 
 
     /*!
@@ -245,8 +294,9 @@ public:
     coefficients at the given energy as a map where the keys are the different physical processes and the
     values are the calculated values via log-log interpolation in the internal table.    */
     std::map<std::string, double> getMassAttenuationCoefficients(\
-                                                std::map<std::string, double> elementMassFractions,\
-                                                double energies) const;
+                                                const std::map<std::string, double> & elementMassFractions,\
+                                                const double & energies,
+                                                const int & isComposition = 0) const;
 
     // Material handling
     /*!
@@ -334,6 +384,15 @@ public:
     std::map<std::string, double> getComposition(const std::string & name, \
                                                  const std::vector<Material> & materials) const;
 
+    /*!
+    Try to interpret a composition (in terms of map of strings representing elements, chemical formulas or material names)
+    and mass fractions using the supplied supplied list of materials.
+    It returns the composition in terms of elements and mass fractions.
+    In case of failure, it returns an empty map.
+    */
+    std::map<std::string, double> getComposition(const std::map<std::string, double> & composition, \
+                                                 const std::vector<Material> & materials = std::vector<Material>()) const;
+
     /*!
     Try to parse a given string as a formula, returning the associated number of "atoms"
     per single molecule. In case of failure, it returns an empty map.
@@ -351,7 +410,7 @@ public:
                             const std::vector<double> & weights = std::vector<double>()) const;
 
     /*!
-    Given an energy and its (optional) weight, for the specfified element, this method returns
+    Given an energy and its (optional) weight, for the specified element, this method returns
     the emitted X-ray already corrected for cascade and fluorescence yield.
     It is the equivalent of the excitation factor in D.K.G. de Boer's paper.
     */
@@ -555,7 +614,7 @@ private:
                                         const double & thickness) const;
 
     struct sortVectorOfExcited {
-        bool operator()(const std::pair<std::string, double> &left, const std::pair<std::string,int> &right) {
+        bool operator()(const std::pair<std::string, double> &left, const std::pair<std::string, double> &right) {
         return left.second < right.second;
     }
 };
