@@ -2,7 +2,7 @@
 #
 # The fisx library for X-Ray Fluorescence
 #
-# Copyright (c) 2014-2020 European Synchrotron Radiation Facility
+# Copyright (c) 2014-2023 European Synchrotron Radiation Facility
 #
 # This file is part of the fisx X-ray developed by V.A. Sole
 #
@@ -33,6 +33,7 @@
 #include <iostream>
 #include <sstream>
 #include <iomanip>
+#include <algorithm>
 
 namespace fisx
 {
@@ -76,10 +77,10 @@ void XRF::setBeam(const Beam & beam)
     this->configuration.setBeam(beam);
 }
 
-void XRF::setBeam(const double & energy, const double & divergency)
+void XRF::setSingleEnergyBeam(const double & energy, const double & divergency)
 {
     this->recentBeam = true;
-    this->configuration.setBeam(energy, divergency);
+    this->configuration.setSingleEnergyBeam(energy, divergency);
 }
 
 void XRF::setBeam(const std::vector<double> & energies, \
@@ -197,7 +198,8 @@ std::map<std::string, std::map<int, std::map<std::string, std::map<std::string,
                 const Elements & elementsLibrary, const int & sampleLayerIndex, \
                 const std::string & lineFamily, const int & secondary, \
                 const int & useGeometricEfficiency, const int & useMassFractions, \
-                const double & optimizationFactor )
+                const double & optimizationFactor, \
+                const Beam & overwritingBeam )
 {
     std::vector<std::string> elementList;
     std::vector<std::string> familyList;
@@ -219,7 +221,7 @@ std::map<std::string, std::map<int, std::map<std::string, std::map<std::string,
     layerList.push_back(sampleLayerIndex);
     return this->getMultilayerFluorescence(elementList, elementsLibrary, layerList, familyList, \
                                            secondary, useGeometricEfficiency, useMassFractions, \
-                                           optimizationFactor);
+                                           optimizationFactor, overwritingBeam);
 }
 
 double XRF::getEnergyThreshold(const std::string & elementName, const std::string & family, \
@@ -260,11 +262,141 @@ const XRFConfig & XRF::getConfiguration() const
 
 }
 
+std::map<std::string, double> XRF::getLayerComposition(const Layer & layer, const Elements & elements) const
+{
+    std::map <std::string, double> composition;
+    Material layerMaterial;
+
+    if (layer.hasMaterialComposition())
+    {
+        // It has an actual material with the composition. However, the name of the material can be unset
+        // and one cannot simply use its name via the line commented below.
+        // return elements.getComposition(layer.getMaterialName(), this->getMaterials());
+        return elements.getComposition(layer.getMaterial().getComposition(), this->getMaterials());
+    }
+    else
+    {
+        // It has a material name, element or formula as composition
+        return elements.getComposition(layer.getMaterialName(), this->getMaterials());
+    }
+}
+
+std::map<std::string, double> XRF::getLayerMassAttenuationCoefficients( \
+                                                const Layer & layer,
+                                                const double & energy,
+                                                const Elements & elements,
+                                                const std::map<std::string, double> & layerComposition) const
+{
+    if (layerComposition.size() > 0)
+        return elements.getMassAttenuationCoefficients(layerComposition, energy, 1);
+    else
+        return elements.getMassAttenuationCoefficients(this->getLayerComposition(layer, elements), energy);
+}
+
+std::map<std::string, std::vector<double> > XRF::getLayerMassAttenuationCoefficients(const Layer & layer,
+                                                const std::vector<double> & energy,
+                                                const Elements & elements,
+                                                const std::map<std::string, double> & layerComposition) const
+{
+    if (layerComposition.size() > 0)
+    {
+        // this is an optimization path where the layer composition is already in terms of elements
+        // and not in terms of materials or formulas to be parsed
+        return elements.getMassAttenuationCoefficients(layerComposition, energy, 1);
+    }
+    else
+    {
+        return elements.getMassAttenuationCoefficients(this->getLayerComposition(layer, elements), energy);
+    }
+}
+
+double XRF::getLayerTransmission(const Layer & layer,
+                                 const double & energy,
+                                 const Elements & elements,
+                                 const double & angle,
+                                 const std::map<std::string, double> & layerComposition) const
+{
+    std::vector<double> energies;
+
+    energies.resize(0);
+    energies.push_back(energy);
+    return this->getLayerTransmission(layer, energies, elements, angle, layerComposition)[0];
+}
+
+std::vector<double> XRF::getLayerTransmission(const Layer & layer,
+                                             const std::vector<double> & energy,
+                                             const Elements & elements,
+                                             const double & angle,
+                                             const std::map<std::string, double> & layerComposition) const
+{
+    const double PI = std::acos(-1.0);
+    std::vector<double>::size_type i;
+    std::vector<double> tmpDoubleVector;
+    double tmpDouble;
+
+    if (angle == 90.0)
+    {
+        tmpDouble = layer.getDensity() * layer.getThickness();
+    }
+    else
+    {
+        if (angle < 0)
+            tmpDouble = std::sin(-angle * PI / 180.);
+        else
+            tmpDouble = std::sin(angle * PI / 180.);
+        tmpDouble = layer.getDensity() * layer.getThickness() / tmpDouble;
+    }
+
+    if(tmpDouble <= 0.0)
+    {
+        std::string msg;
+        msg = "Layer " + layer.getName() + " thickness is " + elements.toString(tmpDouble) + " g/cm2";
+        throw std::runtime_error( msg );
+    }
+
+    tmpDoubleVector = this->getLayerMassAttenuationCoefficients(layer, energy, elements, layerComposition)["total"];
+
+    for (i = 0; i < tmpDoubleVector.size(); i++)
+    {
+        tmpDoubleVector[i] = (1.0 - layer.getFunnyFactor()) + \
+                              (layer.getFunnyFactor() * exp(-(tmpDouble * tmpDoubleVector[i])));
+    }
+    return tmpDoubleVector;
+}
+
+std::vector<std::pair<std::string, double> > XRF::getLayerPeakFamilies( \
+                                                        const Layer & layer,
+                                                        const double & energy,
+                                                        const Elements & elements,
+                                                        const std::map<std::string, double> & layerComposition) const
+{
+    std::map<std::string, double> composition;
+    std::vector<std::string> elementsList;
+    std::map<std::string, double>::const_iterator c_it;
+
+    if (layerComposition.size() > 0)
+        composition = layerComposition;
+    else
+        composition = this->getLayerComposition(layer, elements);
+
+    for(c_it = composition.begin(); c_it != composition.end(); ++c_it)
+    {
+        if (std::find(elementsList.begin(), elementsList.end(), c_it->first) == elementsList.end())
+        {
+            elementsList.push_back(c_it->first);
+        }
+    }
+
+    return elements.getPeakFamilies(elementsList, energy);
+
+}
+
+
 std::map<std::string, std::map<int, std::map<std::string, std::map<std::string, double> > > > \
                 XRF::getMultilayerFluorescence(const std::vector<std::string> & elementFamilyLayer, \
                 const Elements & elementsLibrary, const int & secondary, \
                 const int & useGeometricEfficiency, const int & useMassFractions, \
-                const double & secondaryCalculationLimit)
+                const double & secondaryCalculationLimit, const Beam & overwritingBeam)
 {
     std::vector<std::string> elementList;
     std::vector<std::string> familyList;
@@ -310,7 +442,12 @@ std::map<std::string, std::map<int, std::map<std::string, std::map<std::string,
     }
     return this->getMultilayerFluorescence(elementList, elementsLibrary, \
                                            layerList, familyList, secondary, useGeometricEfficiency, \
-                                           useMassFractions, secondaryCalculationLimit);
+                                           useMassFractions, secondaryCalculationLimit, overwritingBeam);
+}
+
+void XRF::printConfiguration() const
+{
+    std::cout << this->getConfiguration() << std::endl;
 }
 
 } // namespace fisx
